@@ -1,15 +1,20 @@
+# -*- coding: utf-8 -*-
 """SparkMonitor IPython Kernel Extension
 
 Receives data from listener and forwards to frontend.
 Adds a configuration object to users namespace.
 """
+from __future__ import absolute_import
+from __future__ import unicode_literals
 
-import socket
 import logging
 import os
 import subprocess
+import socket
 from threading import Thread
 
+import pkg_resources
+
 ipykernel_imported = True
 spark_imported = True
 try:
@@ -20,7 +25,12 @@ except ImportError:
 try:
     from pyspark import SparkConf
 except ImportError:
-    spark_imported = False
+    try:
+        import findspark
+        findspark.init()
+        from pyspark import SparkConf
+    except Exception:
+        spark_imported = False
 
 
 class ScalaMonitor:
@@ -47,33 +57,32 @@ class ScalaMonitor:
         self.comm.send(msg)
 
     def handle_comm_message(self, msg):
-        action = msg['content']['data']['action']
-        if action == 'openMonitor':
-            conf = self.ipython.user_ns.get('swan_spark_conf')
-            port = conf.get('spark.ui.port')
-            if not port:
-                port = 4040
-            self.send({'msgtype': 'openSparkUIFrame', 'port': port, 'url': msg['content']['data']['url']})
+        """Handle message received from frontend
+
+        Does nothing for now as this only works if kernel is not busy.
+        """
+        logger.debug('COMM MESSAGE:  \n %s', str(msg))
 
     def register_comm(self):
-        """Register a comm_target which will be used by frontend to start communication."""
+        """Register a comm_target which will be used by
+        frontend to start communication."""
         self.ipython.kernel.comm_manager.register_target(
-            "SparkMonitor", self.target_func)
+            'SparkMonitor', self.target_func)
 
     def target_func(self, comm, msg):
         """Callback function to be called when a frontend comm is opened"""
-        log.info("Comm opened")
-        log.debug("Comm opened message: %s", str(msg))
+        logger.info('SparkMonitor comm opened from frontend.')
         self.comm = comm
 
         @self.comm.on_msg
         def _recv(msg):
             self.handle_comm_message(msg)
-        comm.send({"msgtype": "commopen"})
+        comm.send({'msgtype': 'commopen'})
 
 
 class SocketThread(Thread):
-    """Class to manage a socket in a background thread to talk to the scala listener."""
+    """Class to manage a socket in a background thread
+    to talk to the scala listener."""
 
     def __init__(self):
         """Constructor, initializes base class Thread."""
@@ -81,12 +90,13 @@ class SocketThread(Thread):
         Thread.__init__(self)
 
     def startSocket(self):
-        """Starts a socket on a random port and starts listening for connections"""
+        """Starts a socket on a random port and starts
+        listening for connections"""
         self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self.sock.bind(("localhost", self.port))
+        self.sock.bind(('localhost', self.port))
         self.sock.listen(5)
         self.port = self.sock.getsockname()[1]
-        log.info("Socket Listening on port %s", str(self.port))
+        logger.info('Socket Listening on port %s', str(self.port))
         self.start()
         return self.port
 
@@ -97,26 +107,28 @@ class SocketThread(Thread):
         When a connection is closed, goes back into waiting.
         """
         while(True):
-            log.info("Starting socket thread, going to accept")
+            logger.info('Starting socket thread, going to accept')
             (client, addr) = self.sock.accept()
-            log.info("Client Connected %s", addr)
-            totalMessage = ""
+            logger.info('Client Connected %s', addr)
+            totalMessage = ''
             while True:
                 messagePart = client.recv(4096)
                 if not messagePart:
-                    log.info("Scala socket closed - empty data")
+                    logger.info('Scala socket closed - empty data')
                     break
                 totalMessage += messagePart.decode()
                 # Messages are ended with ;EOD:
-                pieces = totalMessage.split(";EOD:")
+                pieces = totalMessage.split(';EOD:')
                 totalMessage = pieces[-1]
                 messages = pieces[:-1]
                 for msg in messages:
-                    log.debug("Message Received: \n%s\n", msg)
+                    logger.debug('Message Received: \n%s\n', msg)
                     self.onrecv(msg)
-            log.info("Socket Exiting Client Loop")
-            client.shutdown(socket.SHUT_RDWR)
-            client.close()
+            logger.info('Socket Exiting Client Loop')
+            try:
+                client.shutdown(socket.SHUT_RDWR)
+            except OSError:
+                client.close()
 
     def start(self):
         """Starts the socket thread"""
@@ -129,8 +141,8 @@ class SocketThread(Thread):
     def onrecv(self, msg):
         """Forwards all messages to the frontend"""
         sendToFrontEnd({
-            "msgtype": "fromscala",
-            "msg": msg
+            'msgtype': 'fromscala',
+            'msg': msg
         })
 
 
@@ -141,21 +153,22 @@ def load_ipython_extension(ipython):
     """
     global ip, monitor  # For Debugging
 
-    global log
-    log = logging.getLogger('tornado.sparkmonitor.kernel')
-    log.name = "SparkMonitorKernel"
-    log.setLevel(logging.INFO)
-    log.propagate = True
+    global logger
+    logger = logging.getLogger('tornado.sparkmonitor.kernel')
+    logger.name = 'SparkMonitorKernel'
+    logger.setLevel(logging.INFO)
+    logger.propagate = True
 
     if ipykernel_imported:
         if not isinstance(ipython, zmqshell.ZMQInteractiveShell):
-            log.warn("Ipython not running through notebook. So exiting.")
+            logger.warn(
+                'SparkMonitor: Ipython not running through notebook')
             return
     else:
         return
 
     ip = ipython
-    log.info("Starting Kernel Extension")
+    logger.info('Starting Kernel Extension')
     monitor = ScalaMonitor(ip)
     monitor.register_comm()  # Communication to browser
     monitor.start()
@@ -163,37 +176,45 @@ def load_ipython_extension(ipython):
     # Injecting conf into users namespace
     if spark_imported:
         # Get conf if user already has a conf for appending
-        conf = ipython.user_ns.get("swan_spark_conf")
+        conf = ipython.user_ns.get('conf')
         if conf:
-            log.info("Conf: " + conf.toDebugString())
+            logger.info('Conf: ' + conf.toDebugString())
             if isinstance(conf, SparkConf):
                 configure(conf)
         else:
             conf = SparkConf()  # Create a new conf
             configure(conf)
-            ipython.push({"swan_spark_conf": conf})  # Add to users namespace
+            ipython.push({
+                'conf': conf, 
+                'swan_spark_conf': conf # For backward compatibility with fork
+                })  # Add to users namespace
 
 
 def configure(conf):
     """Configures the provided conf object.
 
     Sets the Java Classpath and listener jar file path to "conf".
-    Also sets an environment variable for ports for communication with scala listener.
+    Also sets an environment variable for ports for communication
+    with scala listener.
     """
     global monitor
     port = monitor.getPort()
-    log.info("SparkConf Configured, Starting to listen on port: %s", str(port))
-    os.environ["SPARKMONITOR_KERNEL_PORT"] = str(port)
-    log.info(os.environ["SPARKMONITOR_KERNEL_PORT"])
-    conf.set("spark.extraListeners",
-             "sparkmonitor.listener.JupyterSparkMonitorListener")
-    spark_version = get_spark_version()
-    if "2.11" in spark_version:
+    logger.info('SparkConf Configured, Starting to listen on port:', str(port))
+    os.environ['SPARKMONITOR_KERNEL_PORT'] = str(port)
+    logger.info(os.environ['SPARKMONITOR_KERNEL_PORT'])
+    spark_scala_version = get_spark_scala_version()
+    if "2.11" in spark_scala_version:
         jarpath = os.path.abspath(os.path.dirname(__file__)) + "/listener_2.11.jar"
-    elif "2.12" in spark_version:
+        logger.info('Adding jar from %s ', jarpath)
+        conf.set('spark.driver.extraClassPath', jarpath)
+        conf.set('spark.extraListeners', 'sparkmonitor.listener.JupyterSparkMonitorListener')
+    elif "2.12" in spark_scala_version:
         jarpath = os.path.abspath(os.path.dirname(__file__)) + "/listener_2.12.jar"
-    log.info("Adding jar from %s ", jarpath)
-    conf.set("spark.driver.extraClassPath", jarpath)
+        logger.info('Adding jar from %s ', jarpath)
+        conf.set('spark.driver.extraClassPath', jarpath)
+        conf.set('spark.extraListeners', 'sparkmonitor.listener.JupyterSparkMonitorListener')
+    else:
+        logger.warn("Unknown scala version skipped configuring listener jar.")
 
 
 def sendToFrontEnd(msg):
@@ -201,7 +222,7 @@ def sendToFrontEnd(msg):
     global monitor
     monitor.send(msg)
 
-def get_spark_version():
+def get_spark_scala_version():
     cmd = "pyspark --version 2>&1 | grep -m 1  -Eo '[0-9]*[.][0-9]*[.][0-9]*[,]' | sed 's/,$//'"
     version = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")
     return version.stdout.strip()
